﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace IconMaker
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {

            if (saveFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                string savepath = "icon.ico";
                savepath = saveFileDialog1.FileName;

                //Bitmap b256 = (Bitmap)Bitmap.FromFile(txtPath256.Text);
                Bitmap b48 = (Bitmap)Bitmap.FromFile(txtPath48.Text);
                Bitmap b32 = (Bitmap)Bitmap.FromFile(txtPath32.Text);
                Bitmap b16 = (Bitmap)Bitmap.FromFile(txtPath16.Text);

                BinaryWriter bw = new BinaryWriter(File.Open(savepath, FileMode.OpenOrCreate));

                byte[] barr = File.ReadAllBytes(txtPath256.Text);

                // reserved 0x00
                bw.Write((short)0);

                // type. icon = 1
                bw.Write((short)1);

                // number of images
                int numImages = 4;
                bw.Write((short)numImages);

                int offset = 6 + numImages * 16;
                offset = WriteDictionary(bw, (byte)48, (byte)48, (byte)0, (short)32, 0, offset);
                offset = WriteDictionary(bw, (byte)32, (byte)32, (byte)0, (short)32, 0, offset);
                offset = WriteDictionary(bw, (byte)16, (byte)16, (byte)0, (short)32, 0, offset);
                offset = WriteDictionary(bw, (byte)0, (byte)0, (byte)0, (short)32, barr.Length, offset);

                WriteBitmap(bw, b48);
                WriteBitmap(bw, b32);
                WriteBitmap(bw, b16);

                // write the png file
                bw.Write(barr);

                bw.Write(barr);
                bw.Flush();
                bw.Close();
            }

        }

        private int WriteDictionary(BinaryWriter bw, byte width, byte height, byte color_count, short bitsPerPixel, int size, int offset)
        {
            if (size == 0)
            {
                size = 40 + width*height*4 + getFauxSize(width,height);
            }

            bw.Write(width);
            bw.Write(height);
            bw.Write(color_count);
            bw.Write((byte)0); //reserved
            bw.Write((short)1); // color planes
            bw.Write(bitsPerPixel);
            bw.Write(size); // size in bytes
            bw.Write(offset);

            return offset+size;
        }

        private void WriteBitmap(BinaryWriter bw, Bitmap data)
        {
            int width = data.Width;
            int height = data.Height;

            bw.Write((int)40); // header size
            bw.Write(width);
            bw.Write(2*height);
            bw.Write((short)1); // color planes
            bw.Write((short)32); // bpp
            bw.Write((int)0); // compression method - 0 = uncompressed
            bw.Write((int)(width*height*4 + getFauxSize(width,height))); // raw bitmap data size
            bw.Write((int)2835); // horizontal resolution in ppm
            bw.Write((int)2835); // vertical resolution in ppm
            bw.Write((int)0); // number of colors in palette
            bw.Write((int)0); // important colors - 0 = all

            // now write bitmap data from (0, height) to (width, 0)
            for (int y = height - 1; y >= 0; y--)
            {
                for (int x = 0; x < width; x++)
                {
                    Color c = data.GetPixel(x, y);
                    bw.Write((byte)c.B);
                    bw.Write((byte)c.G);
                    bw.Write((byte)c.R);
                    bw.Write((byte)c.A);
                }
            }

            // now write a faux 1-bit AND channel (filled with zeroes)
            for (int i = 0; i < getFauxSize(width , height); i++)
            {
                bw.Write((byte)0);
            }
        }

        private static int getFauxSize(int width, int height)
        {
            // get number of bytes per Row
            int numBytesPerRow = width / 8;

            // align it to a 4 byte boundary
            if (numBytesPerRow % 4 > 0)
            {
                numBytesPerRow += (4 - (numBytesPerRow % 4));
            }

            return height * numBytesPerRow;
        }

        private void btnBrowse256_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                this.txtPath256.Text = openFileDialog1.FileName;
            }
        }
        private void btnBrowse48_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                this.txtPath48.Text = openFileDialog1.FileName;
            }
        }
        private void btnBrowse32_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                this.txtPath32.Text = openFileDialog1.FileName;
            }
        }
        private void btnBrowse16_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog(this) == DialogResult.OK)
            {
                this.txtPath16.Text = openFileDialog1.FileName;
            }
        }

        private void linkHelp_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            string msg = "Icon Maker v 0.1 \r\n"
                + "(c) 2009 Sushovan De\r\n"
                + "sushovande@hotmail.com\r\n\r\n"
                + "The 256 x 256 image HAS to be a PNG image, the other sizes may be any alpha-channel supporting format"
                + " that .NET can read. All images are assumed to be 32 bit-depth.";
            MessageBox.Show(msg);
        }
    }
}
